@@ -13,55 +13,11 @@ from opal_common.topics.utils import (
 )
 from opal_client.logger import logger
 from opal_client.config import opal_client_config
-from opal_client.policy.fetcher import policy_fetcher
+from opal_client.policy.fetcher import PolicyFetcher
 from opal_client.policy_store.base_policy_store_client import BasePolicyStoreClient
 from opal_client.policy_store.policy_store_client_factory import DEFAULT_POLICY_STORE_GETTER
 from opal_client.policy.topics import default_subscribed_policy_directories
 
-
-async def update_policy(policy_store: BasePolicyStoreClient, directories: List[str] = None, force_full_update=False):
-    """
-    fetches policy (code, e.g: rego) from backend and stores it in the policy store.
-
-    Args:
-        policy_store (BasePolicyStoreClient, optional): Policy store client to use to store policy code.
-        directories (List[str], optional): specific source directories we want.
-        force_full_update (bool, optional): if true, ignore stored hash and fetch full policy bundle.
-    """
-    directories = directories if directories is not None else default_subscribed_policy_directories()
-    if force_full_update:
-        logger.info("full update was forced (ignoring stored hash if exists)")
-        base_hash = None
-    else:
-        base_hash = await policy_store.get_policy_version()
-
-    if base_hash is None:
-        logger.info("Refetching policy code (full bundle)")
-    else:
-        logger.info("Refetching policy code (delta bundle), base hash: '{base_hash}'", base_hash=base_hash)
-    bundle: Optional[PolicyBundle] = await policy_fetcher.fetch_policy_bundle(directories, base_hash=base_hash)
-    if bundle:
-        if bundle.old_hash is None:
-            logger.info(
-                "got policy bundle, commit hash: '{commit_hash}'",
-                commit_hash=bundle.hash,
-                manifest=bundle.manifest
-            )
-        else:
-            deleted_files = None if bundle.deleted_files is None else bundle.deleted_files.dict()
-            logger.info(
-                "got policy bundle (delta): '{diff_against_hash}' -> '{commit_hash}'",
-                commit_hash=bundle.hash,
-                diff_against_hash=bundle.old_hash,
-                manifest=bundle.manifest,
-                deleted=deleted_files
-            )
-        # store policy bundle in OPA cache
-        # We wrap our interaction with the policy store with a transaction, so that
-        # if the write-op fails, we will mark the transaction as failed.
-        async with policy_store.transaction_context(bundle.hash) as store_transaction:
-            await store_transaction.set_policies(bundle)
-
 class PolicyUpdater:
     """
     Keeps policy-stores (e.g. OPA) up to date with relevant policy code
@@ -109,6 +65,8 @@ class PolicyUpdater:
         # The task running the Pub/Sub subcribing client
         self._subscriber_task = None
         self._stopping = False
+        # policy fetcher - fetches policy bundles
+        self._policy_fetcher = PolicyFetcher()
 
     async def __aenter__(self):
         await self.start()
@@ -136,7 +94,7 @@ class PolicyUpdater:
             directories = default_subscribed_policy_directories()
             logger.warning("Received policy updated (invalid topic): {topic}", topic=topic)
 
-        await update_policy(self._policy_store, directories)
+        await self.update_policy(directories)
 
     async def _on_connect(self, client: PubSubClient, channel: RpcChannel):
         """
@@ -147,7 +105,7 @@ class PolicyUpdater:
         when the connection is lost we assume we need to start from scratch.
         """
         logger.info("Connected to server")
-        await update_policy(policy_store=self._policy_store)
+        await self.update_policy()
 
     async def _on_disconnect(self, channel: RpcChannel):
         """
@@ -210,4 +168,47 @@ class PolicyUpdater:
             server_uri=self._server_url
         )
         async with self._client:
-            await self._client.wait_until_done()
\ No newline at end of file
+            await self._client.wait_until_done()
+
+    async def update_policy(self, directories: List[str] = None, force_full_update=False):
+        """
+        fetches policy (code, e.g: rego) from backend and stores it in the policy store.
+
+        Args:
+            policy_store (BasePolicyStoreClient, optional): Policy store client to use to store policy code.
+            directories (List[str], optional): specific source directories we want.
+            force_full_update (bool, optional): if true, ignore stored hash and fetch full policy bundle.
+        """
+        directories = directories if directories is not None else default_subscribed_policy_directories()
+        if force_full_update:
+            logger.info("full update was forced (ignoring stored hash if exists)")
+            base_hash = None
+        else:
+            base_hash = await self._policy_store.get_policy_version()
+
+        if base_hash is None:
+            logger.info("Refetching policy code (full bundle)")
+        else:
+            logger.info("Refetching policy code (delta bundle), base hash: '{base_hash}'", base_hash=base_hash)
+        bundle: Optional[PolicyBundle] = await self._policy_fetcher.fetch_policy_bundle(directories, base_hash=base_hash)
+        if bundle:
+            if bundle.old_hash is None:
+                logger.info(
+                    "got policy bundle, commit hash: '{commit_hash}'",
+                    commit_hash=bundle.hash,
+                    manifest=bundle.manifest
+                )
+            else:
+                deleted_files = None if bundle.deleted_files is None else bundle.deleted_files.dict()
+                logger.info(
+                    "got policy bundle (delta): '{diff_against_hash}' -> '{commit_hash}'",
+                    commit_hash=bundle.hash,
+                    diff_against_hash=bundle.old_hash,
+                    manifest=bundle.manifest,
+                    deleted=deleted_files
+                )
+            # store policy bundle in OPA cache
+            # We wrap our interaction with the policy store with a transaction, so that
+            # if the write-op fails, we will mark the transaction as failed.
+            async with self._policy_store.transaction_context(bundle.hash) as store_transaction:
+                await store_transaction.set_policies(bundle)
\ No newline at end of file
