@@ -1,4 +1,4 @@
 from .mock_policy_store_client import MockPolicyStoreClient
 from .opa_client import OpaClient
 from .base_policy_store_client import BasePolicyStoreClient
-from .policy_store_client_factory import PolicyStoreClientFactory, DEFAULT_POLICY_STORE, PolicyStoreTypes
\ No newline at end of file
+from .policy_store_client_factory import PolicyStoreClientFactory, DEFAULT_POLICY_STORE_GETTER, PolicyStoreTypes
\ No newline at end of file
